--+--------------------------------------------------------------------------+
--| IBM Confidential                                                         |
--| OCO Source Materials                                                     |
--| Rational Apex                                                            |
--| (C) Copyright IBM Corp. 2008 All Rights Reserved                         |
--|                                                                          |
--| The source code for this program is not published or otherwise divested  |
--| of its trade secrets, irrespective of what has been deposited with the   |
--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
with Unix_Operations_Extension;
with Tms_Communications;

procedure Import_Server_Port (Server : out Apex_Unix_Types.Int;
                              Port   : out Apex_Unix_Types.Int) is
    --
    --[internal] -- We cannot (should not) use Tm_Environment to 
    --[internal] -- provide Getenv functionality,
    --[internal] -- since it will bloat the users's executable 
    --[internal] -- and the API with the entire Log/Error closure!
    --
    C_Port_Id_Env : constant String          :=
       Tms_Communications.Port_Id_Env & Ascii.Nul;
    Env_Ptr       : Apex_Unix_Types.Char_Ptr :=
       Unix_Operations_Extension.Getenv
          (Apex_Unix_Types.To_Char_Ptr
              (C_Port_Id_Env (C_Port_Id_Env'First)'Address));

    function "=" (L, R : Apex_Unix_Types.Char_Ptr) return Boolean
       renames Apex_Unix_Types."=";

    function Locate (Fragment : Character; Within : String) return Natural is
    begin
        for I in Within'Range loop
            if Fragment = Within (I) then
                return I;
            end if;
        end loop;
    end Locate;


begin
    if Env_Ptr = null then
        Server := 0;
        Port   := 0;
    else
        declare
            Env_Val : constant String  := Apex_Unix_Types.To_String (Env_Ptr);
            Colon   : constant Natural := Locate (':', Env_Val);
            S       : constant String  := Env_Val (Env_Val'First .. Colon - 1);
            P       : constant String  := Env_Val (Colon + 1 .. Env_Val'Last);
        begin
            Server := Apex_Unix_Types.Int'Value (S);
            Port   := Apex_Unix_Types.Int'Value (P);
        end;
    end if;
end Import_Server_Port;
