--+--------------------------------------------------------------------------+
--| IBM Confidential                                                         |
--| OCO Source Materials                                                     |
--| Rational Apex                                                            |
--| (C) Copyright IBM Corp. 2008 All Rights Reserved                         |
--|                                                                          |
--| The source code for this program is not published or otherwise divested  |
--| of its trade secrets, irrespective of what has been deposited with the   |
--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--

with System;
with Tm_Simple_Io;
with Unchecked_Conversion;

package body Tms_Arguments is

    function "=" (L, R : System.Address) return Boolean renames System."=";

    procedure Err_Message (Msg : String) is
    begin
        Tm_Simple_Io.Put_Line ("%%% " & Msg, Tm_Simple_Io.Stderr);
    end Err_Message;

    function To_Size_Array (Value : Natural) return Byte_Array is
        Result : Byte_Array (1 .. 2) := (Apex_Unix_Types.Char (Value / 128),
                                         Apex_Unix_Types.Char (Value rem 128));
    begin
        return Result;
    end To_Size_Array;

    function From_Size_Array (Arr : Byte_Array) return Natural is
        Result : Natural := (Natural (Arr (Arr'First)) * 128) +
                               Natural (Arr (Arr'Last));
    begin
        return Result;
    end From_Size_Array;


    ------------------------ interface routines --------------------------

    function Encode_Routine (Routine : Routine_Names) return Byte_Array is
    begin
        return To_Size_Array (Routine_Names'Pos (Routine));
    end Encode_Routine;

    function Encode (Param : Parameter_Type) return Byte_Array is

        type Acc_Parameter_Type is access Parameter_Type;

        function Dope_Size (Request : Parameter_Type) return Natural is
        begin
            if not Acc_Parameter_Type'Homogeneous then
                -- This check is necessary; SWB assures me there are cases
                -- where there is no dope vector, yet 'Dope_Size is nonzero!
                return ((Request'Dope_Size + 7) / 8);
            else
                return 0;
            end if;
        end Dope_Size;

        function Byte_Size (Request : Parameter_Type) return Natural is
        begin
            return ((Request'Size + 7) / 8);
        end Byte_Size;

        function Build (Request : Parameter_Type) return Byte_Array is
            Base_Size        : Natural := Byte_Size (Request);
            Dope_Vector_Size : Natural := Dope_Size (Request);
            Result           : Byte_Array (1 .. Base_Size + Dope_Vector_Size);

            procedure Copy (Source, Target : System.Address; Count : Natural) is
                subtype Node is Byte_Array (1 .. Count);
                type Node_Pointer is access Node;
                for Node_Pointer'Storage_Size use 0;

                function Convert is new Unchecked_Conversion
                                           (System.Address, Node_Pointer);
                X : Node_Pointer := Convert (Target);
                S : Node_Pointer := Convert (Source);
            begin
                if Count /= 0 then
                    X.all := S.all;
                end if;
            end Copy;

        begin
            if Dope_Vector_Size > 0 then
                if Request'Dope_Address = System.Null_Address then
                    Copy (Source => Parameter_Type'Dope_Address,
                          Target => Result (Result'First)'Address,
                          Count  => Dope_Vector_Size);
                else
                    Copy (Source => Request'Dope_Address,
                          Target => Result (Result'First)'Address,
                          Count  => Dope_Vector_Size);
                end if;
            end if;
            if Base_Size > 0 then
                Copy (Source => Request'Address,
                      Target => Result (Dope_Vector_Size + 1)'Address,
                      Count  => Base_Size);
            end if;

            return To_Size_Array (Result'Length) & Result;
        end Build;

    begin
        return Build (Request => Param);
    end Encode;

    function Encode_String (Param : String) return Byte_Array is

        type Acc_Parameter_Type is access String;

        function Dope_Size (Request : String) return Natural is
        begin
            if not Acc_Parameter_Type'Homogeneous then
                -- This check is necessary; SWB assures me there are cases
                -- where there is no dope vector, yet 'Dope_Size is nonzero!
                return ((Request'Dope_Size + 7) / 8);
            else
                return 0;
            end if;
        end Dope_Size;

        function Byte_Size (Request : String) return Natural is
        begin
            return ((Request'Size + 7) / 8);
        end Byte_Size;

        function Build (Request : String) return Byte_Array is
            Base_Size        : Natural := Byte_Size (Request);
            Dope_Vector_Size : Natural := Dope_Size (Request);
            Result           : Byte_Array (1 .. Base_Size + Dope_Vector_Size);

            procedure Copy (Source, Target : System.Address; Count : Natural) is
                subtype Node is Byte_Array (1 .. Count);
                type Node_Pointer is access Node;
                for Node_Pointer'Storage_Size use 0;

                function Convert is new Unchecked_Conversion
                                           (System.Address, Node_Pointer);

                S : Node_Pointer := Convert (Source);
                T : Node_Pointer := Convert (Target);
                pragma Suppress_All;
            begin
                if Count /= 0 then
                    T.all := S.all;
                    --Convert (Target).all := Convert (Source).all;
                end if;
            end Copy;

        begin
            if Dope_Vector_Size > 0 then
                if Request'Dope_Address = System.Null_Address then
                    Copy (Source => String'Dope_Address,
                          Target => Result (Result'First)'Address,
                          Count  => Dope_Vector_Size);
                else
                    Copy (Source => Request'Dope_Address,
                          Target => Result (Result'First)'Address,
                          Count  => Dope_Vector_Size);
                end if;
            end if;
            if Base_Size > 0 then
                Copy (Source => Request'Address,
                      Target => Result (Dope_Vector_Size + 1)'Address,
                      Count  => Base_Size);
            end if;

            return To_Size_Array (Result'Length) & Result;
        end Build;

    begin
        return Build (Request => Param);
    end Encode_String;

    function Routine (From_Call : Byte_Array) return Routine_Names is
    begin
        return Routine_Names'Val
                  (From_Size_Array (From_Call (From_Call'First ..
                                                  From_Call'First + 1)));
    end Routine;

    function Parameter_Count (From_Call : Byte_Array) return Natural is
        Next  : Natural := From_Call'First + 2;
        Count : Natural := 0;
    begin
        while Next < From_Call'Last loop
            -- increment past size_array + parameter_size
            Next  := Next + 2 + From_Size_Array (From_Call (Next .. Next + 1));
            Count := Count + 1;
        end loop;
        return Count;
    end Parameter_Count;

    function Find_Parameter
                (From_Call : Byte_Array; Param : Natural) return Natural is
        Next  : Natural := From_Call'First + 2;
        Count : Natural := 1;
    begin
        while Param /= Count loop
            -- increment past size_array + parameter_size
            Next  := Next + 2 + From_Size_Array (From_Call (Next .. Next + 1));
            Count := Count + 1;
        end loop;
        return Next;
    end Find_Parameter;

    function Decode (From_Call : Byte_Array; Parameter : Natural)
                    return Parameter_Type is

        Param_Loc  : Natural                      :=
           Find_Parameter (From_Call, Parameter);
        Param_Size : Natural                      :=
           From_Size_Array (From_Call (Param_Loc .. Param_Loc + 1));
        -- force array to an aligned word boundry ...
        Param      : Byte_Array (1 .. Param_Size) :=
           From_Call (Param_Loc + 2 .. Param_Loc + 2 + Param_Size - 1);

        type Acc_Parameter_Type is access Parameter_Type;

        function Convert is new Unchecked_Conversion
                                   (System.Address, Acc_Parameter_Type);

    begin
        if Param_Size /= 0 then
            declare
                Result : constant Parameter_Type :=
                   Convert (Param (1)'Address).all;
            begin
                return Result;
            end;
        else
            return Nil_Parameter_Type;
        end if;
    exception
        when others =>
            Err_Message ("ERROR Reverting Byte Array in " &
                         Routine_Names'Image (Routine (From_Call)));
    end Decode;

    function Decode_String
                (From_Call : Byte_Array; Parameter : Natural) return String is

        Param_Loc  : Natural                      :=
           Find_Parameter (From_Call, Parameter);
        Param_Size : Natural                      :=
           From_Size_Array (From_Call (Param_Loc .. Param_Loc + 1));
        -- force array to an aligned word boundry ...
        Param      : Byte_Array (1 .. Param_Size) :=
           From_Call (Param_Loc + 2 .. Param_Loc + 2 + Param_Size - 1);

        type Acc_Parameter_Type is access String;

        function Convert is new Unchecked_Conversion
                                   (System.Address, Acc_Parameter_Type);

    begin
        if Param_Size /= 0 then
            declare
                Result : constant String := Convert (Param (1)'Address).all;
            begin
                return Result;
            end;
        else
            return "";
        end if;
    exception
        when others =>
            Err_Message ("ERROR Reverting Byte Array in " &
                         Routine_Names'Image (Routine (From_Call)));
    end Decode_String;
end Tms_Arguments;

