--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
with Tms_Arguments;
with Tms_Communications;
with Tms_Communications_Client;
with Unchecked_Deallocation;

package body Tms_Script_Commands is

    package Ta renames Tms_Arguments;
    package Tc renames Tms_Communications;
    package Tcc renames Tms_Communications.Communicate;
    package Client renames Tms_Communications_Client;

    function "&" (L, R : Ta.Byte_Array) return Ta.Byte_Array renames Ta."&";

    function Encode_Tc_Termination_Reason is
       new Ta.Encode (Tc_Termination_Reason);

    function Encode_String (Param : String) return Ta.Byte_Array
       renames Ta.Encode_String;
    function Decode_String
		(From_Call : Ta.Byte_Array; Parameter : Natural) return String
       renames Ta.Decode_String;

    function Decode_Seconds is new Ta.Decode (Seconds, 0);
    function Decode_Boolean is new Ta.Decode (Boolean, False);
    function Decode_Natural is new Ta.Decode (Natural, 0);
    function Decode_Termination_Reason is
       new Ta.Decode (Termination_Reason, None);

    function Create (Str : String) return String_Handle is
    begin
	return new String'(Str);
    end Create;

    procedure Destroy is new Unchecked_Deallocation (String, String_Handle);

    procedure Set (The_Handle : in out String_Handle; To : String) is
    begin
	if The_Handle /= null then
	    Destroy (The_Handle);
	end if;
	The_Handle := Create (To);
    end Set;


    function Locate (The_Char : Character; In_String : String) return Integer is
    begin
	for I in In_String'Range loop
	    if In_String (I) = The_Char then
		return I;
	    end if;
	end loop;
	return 0;
    end Locate;


    function Resolve_Placeholders (In_The_String : String) return String is
	Request : constant Ta.Byte_Array :=
	   Ta.Encode_Routine (Ta.Resolve_Placeholder) &
	      Encode_String (In_The_String);
    begin
	if Locate ('<', In_The_String) = 0 or else
	   Locate ('>', In_The_String) = 0 or else Client.Bastard then
	    return In_The_String;
	end if;

	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array := Tcc.Receive;
	begin
	    return Decode_String (Response, 1);
	end;
    exception
	when others =>
	    Tc.Report_Error ("Could not resolve placeholders in """ &
			     In_The_String & '"');
	    if Client.Bastard then
		return In_The_String;
	    else
		raise;
	    end if;
    end Resolve_Placeholders;


    function Coverage_Pre_Run return String is
	Request : constant Ta.Byte_Array :=
	   Ta.Encode_Routine (Ta.Coverage_Pre_Run);
    begin
	if Client.Bastard then
	    return "";
	end if;

	-- Write to parent (and return) ...
	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    return Decode_String (Response, 1);
	end;
    exception
	when others =>
	    Tc.Report_Error ("Could not perform coverage pre run");
	    if not Client.Bastard then
		raise;
	    else
		return "";
	    end if;
    end Coverage_Pre_Run;


    procedure Coverage_Post_Run is
	Request : constant Ta.Byte_Array :=
	   Ta.Encode_Routine (Ta.Coverage_Post_Run);
    begin
	if Client.Bastard then
	    return;
	end if;

	-- Write to parent (and return) ...
	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    null;
	end;
    exception
	when others =>
	    Tc.Report_Error ("Could not perform coverage post run");
	    if not Client.Bastard then
		raise;
	    end if;
    end Coverage_Post_Run;


    function Initialize return Natural is
	Request : constant Ta.Byte_Array :=
	   Ta.Encode_Routine (Ta.Test_List_Initialize);
    begin
	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    if Client.Bastard then
		Tc.Report_Error
		   (Msg           =>
		       "No test execution server (not running under TestMate)",
		    Force_Display => True);
		return 0;
	    end if;
	    return Decode_Natural (Response, 1);
	end;
    exception
	when others =>
	    Tc.Report_Error (Msg           =>
				"Could not initialize test script",
			     Force_Display => True);
	    if not Client.Bastard then
		raise;
	    end if;
	    return 0;
    end Initialize;


    procedure Initialize is
	Discard : Natural := Initialize;
    begin
	null;
    end Initialize;


    procedure Start_Test_Case (Input_File     : in out String_Handle;
			       Output_File    : in out String_Handle;
			       Error_File     : in out String_Handle;
			       Exec_Context   : in out String_Handle;
			       Allow_Coverage : in out Boolean;
			       Cpu_Limit      : in out Seconds;
			       Elapsed_Limit  : in out Seconds;
			       Term_Reason    : in out Termination_Reason) is
	Request : constant Ta.Byte_Array :=
	   Ta.Encode_Routine (Ta.Start_Test_Case);
    begin
	if Client.Bastard then
	    Tc.Report_Error (Msg           => "Start of test case",
			     Force_Display => True);
	    Term_Reason := None;
	    return;
	end if;

	-- Write to parent (and return) ...
	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    Set (Input_File, Decode_String (Response, 1));
	    Set (Output_File, Decode_String (Response, 2));
	    Set (Error_File, Decode_String (Response, 3));
	    Set (Exec_Context, Decode_String (Response, 4));

	    Allow_Coverage := Decode_Boolean (Response, 5);
	    Cpu_Limit      := Decode_Seconds (Response, 6);
	    Elapsed_Limit  := Decode_Seconds (Response, 7);
	    Term_Reason    := Decode_Termination_Reason (Response, 8);
	end;
    exception
	when others =>
	    Tc.Report_Error
	       ("Could not initialize test case; using default values");
	    Term_Reason := None;
    end Start_Test_Case;


    procedure Start_Test_Case is
	--
	-- we ignore all params since this is called from the command-line
	-- The implication here is that all features dependent on the 
	-- correct interpretation of Start_Test_Case params are disabled
	-- for command-line calls (and therefore, since this is the interface
	-- used by TestMate Cross, they are disabled for target Ada test cases)
	--
	-- Features disabled are:
	--
	--  Execution_Context     (cwd for running the driver)
	--  Input file redirection
	--  Output file redirection
	--  CPU limits            (disabled for all ATC/CTC anuway...)
	--  Elapsed limits        (disabled for all ATC/CTC anyway...)
	--  Early script term     (yikes)
	--  Individual control over coverage analysis on ATC/CTC
	--
	Junk1, Junk2, Junk3, Junk4 : String_Handle;
	Junk5                      : Boolean;
	Junk6, Junk7               : Seconds;
	Junk8                      : Termination_Reason;
    begin
	Start_Test_Case (Junk1, Junk2, Junk3, Junk4,
			 Junk5, Junk6, Junk7, Junk8);
    end Start_Test_Case;


    procedure Start_Driver is
	Request : constant Ta.Byte_Array := Ta.Encode_Routine (Ta.Start_Driver);
    begin
	if Client.Bastard then
	    return;
	end if;

	-- Write to parent (and return) ...
	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    null;
	end;
    exception
	when others =>
	    Tc.Report_Error ("Could not synch with coverage analyzer");
	    raise;
    end Start_Driver;


    procedure End_Driver is
	Request : constant Ta.Byte_Array := Ta.Encode_Routine (Ta.End_Driver);
    begin
	if Client.Bastard then
	    return;
	end if;

	-- Write to parent (and return) ...
	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    null;
	end;
    exception
	when others =>
	    Tc.Report_Error ("Could not synch with coverage analyzer");
	    raise;
    end End_Driver;

    procedure End_Test_Case (Term_Reason : in Tc_Termination_Reason) is
	Request : constant Ta.Byte_Array :=
	   Ta.Encode_Routine (Ta.End_Test_Case) &
	      Encode_Tc_Termination_Reason (Term_Reason);
    begin
	if Client.Bastard then
	    return;
	end if;

	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    null;
	end;

    exception
	when others =>
	    Tc.Report_Error ("Could not synch with execution server");
	    raise;
    end End_Test_Case;


    procedure End_Test_Case is
	Junk : Tc_Termination_Reason;
    begin
	End_Test_Case (Junk);
    end End_Test_Case;


    function Getenv (The_Variable : String) return String is
	Request : constant Ta.Byte_Array :=
	   Ta.Encode_Routine (Ta.Getenv) & Encode_String (The_Variable);
    begin
	if Client.Bastard then
	    return "";
	end if;

	-- Write to parent and get response
	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    return Decode_String (Response, 1);
	end;
    exception
	when others =>
	    Tc.Report_Error
	       ("Could not get value of environment variable " & The_Variable);
	    raise;
    end Getenv;


    procedure Defer_Results is
	Request : constant Ta.Byte_Array :=
	   Ta.Encode_Routine (Ta.Defer_Results);
    begin
	if Client.Bastard then
	    return;
	end if;

	-- Write to parent and get response
	Client.Send (Request);
	declare
	    Response : constant Ta.Byte_Array :=
	       Tcc.Receive; -- keep sync'ed with server
	begin
	    null;
	end;
    exception
	when others =>
	    Tc.Report_Error ("Could not defer target results processing");
	    raise;
    end Defer_Results;


end Tms_Script_Commands;

--[api] pragma Api (Testmate);
