--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--

-- Current TestMate licensees may need to customize this code for their 
-- installations.  All other rights reserved.
--

with Text_Io;
with Sequential_Io;
package body Tms_Evals is

    procedure Gang_Result (The_Result : Tms.Test_Result) is
	--
	-- Go ahead and register the result, unless it would cause
	-- a PASS to overwrite an already failing test case
	--
	Current_Result : constant Tms.Test_Result      :=
	   Tms.The_Test_Case_Result ("<test_case>");
	Current_Assert : constant Tms.Assertion_Status := Tms.Check_Assertions;
	New_Result     : Tms.Test_Result renames The_Result;
    begin
	if Current_Result = Tms.Unknown or Current_Result = Tms.Pass then
	    case Current_Assert is
		when Tms.None | Tms.All_True =>
		    Tms.Register_Result (New_Result);
		when Tms.Some_False =>
		    Tms.Register_Result (Tms.Failed_Assertion);
	    end case;
	else
	    if New_Result /= Tms.Pass then
		Tms.Register_Result (New_Result);
	    end if;
	end if;
    end Gang_Result;


    package Byte_Io is new Sequential_Io (Character);

    procedure File_Compare (Expected : String := "<expected_output:f>";
			    Actual   : String := "<output:f>") is
	Expected_File    : constant String :=
	   Tms.Resolve_Placeholders (Expected);
	Actual_File      : constant String := Tms.Resolve_Placeholders (Actual);
	Eo_File, Ao_File : Byte_Io.File_Type;
	Eo_Buff, Ao_Buff : Character;
	Eo_Last, Ao_Last : Natural;
	Byte_Number      : Natural         := 0;
	Quit             : Boolean         := False;
	Eo1F             : Boolean         := False;

	Max_Differences : constant Natural := 5;
	Bytes_Different : Natural range 0 .. Max_Differences;

	procedure Close_Em is
	begin
	    if Byte_Io.Is_Open (Eo_File) then
		Byte_Io.Close (Eo_File);
	    end if;
	    if Byte_Io.Is_Open (Ao_File) then
		Byte_Io.Close (Ao_File);
	    end if;
	end Close_Em;

    begin
	begin
	    Byte_Io.Open (File => Eo_File,
			  Mode => Byte_Io.In_File,
			  Name => Expected_File,
			  Form => "");
	exception
	    when others =>
		Tms.Assert (False, "File opened (could not open " &
				      Expected_File & ")");
		Quit := True;
		Eo1F := True;
	end;

	begin
	    Byte_Io.Open (File => Ao_File,
			  Mode => Byte_Io.In_File,
			  Name => Actual_File,
			  Form => "");
	exception
	    when others =>
		Tms.Assert (False, "File opened (could not open " &
				      Actual_File & ")");
		Quit := True;
		Eo1F := True;
	end;

	Bytes_Different := 0;
	while not Eo1F and then Bytes_Different < Max_Differences loop
	    Byte_Number := Byte_Number + 1;
	    Byte_Io.Read (Eo_File, Eo_Buff);
	    Byte_Io.Read (Ao_File, Ao_Buff);
	    if Eo_Buff /= Ao_Buff then
		Tms.Assert (False, "Byte" & Natural'Image (Byte_Number) &
				      " differs (expected " &
				      Character'Image (Eo_Buff) & ", actual " &
				      Character'Image (Ao_Buff) & ')');
		Bytes_Different := Bytes_Different + 1;
	    end if;
	    Eo1F := Byte_Io.End_Of_File (Eo_File) or
		       Byte_Io.End_Of_File (Ao_File);
	end loop;


	if not Quit then
	    if Eo1F and then

	       (not Byte_Io.End_Of_File (Eo_File) or else
		not Byte_Io.End_Of_File (Ao_File)) then
		Tms.Assert (False, "Files have different sizes");
		Bytes_Different := 1;
	    end if;

	    if Bytes_Different = 0 then
		Tms.Assert (True, "Files match");
	    else
		Tms.Assert (False, "Files do not match ..." & Ascii.Lf &
				      Expected_File & Ascii.Lf & Actual_File);
	    end if;
	end if;
	Close_Em;
    exception
	when others =>
	    Tms.Assert (False,
			"Exception occurred reading files (expected none)");
	    Close_Em;
    end File_Compare;


    procedure Text_File_Compare (Expected : String := "<expected_output:f>";
				 Actual   : String := "<output:f>") is
	Expected_File    : constant String :=
	   Tms.Resolve_Placeholders (Expected);
	Actual_File      : constant String := Tms.Resolve_Placeholders (Actual);
	Eo_File, Ao_File : Text_Io.File_Type;
	Eo_Buff, Ao_Buff : String (1 .. 1024);
	Eo_Last, Ao_Last : Natural;
	Line_Number      : Natural         := 0;
	Quit             : Boolean         := False;
	Eo1F             : Boolean         := False;

	Max_Differences : constant Natural := 3;
	Lines_Different : Natural range 0 .. Max_Differences;
	Max_Diag_Line   : constant         := 70;

	procedure Close_Em is
	begin
	    if Text_Io.Is_Open (Eo_File) then
		Text_Io.Close (Eo_File);
	    end if;
	    if Text_Io.Is_Open (Ao_File) then
		Text_Io.Close (Ao_File);
	    end if;
	end Close_Em;

    begin
	begin
	    Text_Io.Open (File => Eo_File,
			  Mode => Text_Io.In_File,
			  Name => Expected_File,
			  Form => "");
	exception
	    when others =>
		Tms.Assert (False, "File opened (could not open " &
				      Expected_File & ")");
		Quit := True;
		Eo1F := True;
	end;

	begin
	    Text_Io.Open (File => Ao_File,
			  Mode => Text_Io.In_File,
			  Name => Actual_File,
			  Form => "");
	exception
	    when others =>
		Tms.Assert (False, "File opened (could not open " &
				      Actual_File & ")");
		Quit := True;
		Eo1F := True;
	end;

	Lines_Different := 0;
	while not Eo1F and then Lines_Different < Max_Differences loop
	    Line_Number := Line_Number + 1;
	    Text_Io.Get_Line (Eo_File, Eo_Buff, Eo_Last);
	    Text_Io.Get_Line (Ao_File, Ao_Buff, Ao_Last);
	    if Eo_Last /= Ao_Last or else
	       Eo_Buff (1 .. Eo_Last) /= Ao_Buff (1 .. Ao_Last) then
		declare
		    Eo_Start       : Natural                         :=
		       Eo_Buff'First;
		    Eo_End         : Natural                         := Eo_Last;
		    Eo_Pickup      : Character                       := ' ';
		    Eo_Continue    : Character                       := ' ';
		    Ao_Start       : Natural                         :=
		       Ao_Buff'First;
		    Ao_End         : Natural                         := Ao_Last;
		    Ao_Continue    : Character                       := ' ';
		    Ao_Pickup      : Character                       := ' ';
		    First_Mismatch : Natural                         :=
		       Eo_Buff'First;
		    Spaces         : String (1 .. Max_Diag_Line - 1) :=
		       (others => ' ');
		begin
		    while Ao_Buff (First_Mismatch) =
			     Eo_Buff (First_Mismatch) loop
			First_Mismatch := First_Mismatch + 1;
		    end loop;

		    if First_Mismatch > Max_Diag_Line - 10 then
			Ao_Start  := First_Mismatch - 20;
			Eo_Start  := Ao_Start;
			Ao_Pickup := '.';
			Eo_Pickup := '.';
		    end if;

		    if Eo_Last > Eo_Start + (Max_Diag_Line - 1) then
			Eo_End      := Eo_Start + (Max_Diag_Line - 1);
			Eo_Continue := '.';
		    end if;
		    if Ao_Last > Ao_Start + (Max_Diag_Line - 1) then
			Ao_End      := Ao_Start + (Max_Diag_Line - 1);
			Ao_Continue := '.';
		    end if;
		    Tms.Assert
		       (False,
			"Line" & Natural'Image (Line_Number) &
			   " differs at character" &
			   Natural'Image (First_Mismatch) &
			   " (expected identical lines)" &
			   Ascii.Lf & "expected: " &
			   Eo_Buff (Eo_Start .. Eo_End) & Eo_Continue &
			   Eo_Continue & Eo_Continue &
			   Ascii.Lf & "  actual: " &
			   Ao_Buff (Ao_Start .. Ao_End) &
			   Ao_Continue & Ao_Continue &
			   Ao_Continue & Ascii.Lf & "     at->" &
			   Spaces
			      (Spaces'First .. First_Mismatch + 1 - Eo_Start) &
			   '^');
		end;
		Lines_Different := Lines_Different + 1;
	    end if;
	    Eo1F := Text_Io.End_Of_File (Eo_File) or else
		       Text_Io.End_Of_File (Ao_File);
	end loop;

	if not Quit then
	    if Eo1F and then (not Text_Io.End_Of_File (Eo_File) or else
			      not Text_Io.End_Of_File (Ao_File)) then
		Tms.Assert (False, "Files have different sizes");
		Lines_Different := 1;
	    end if;

	    if Lines_Different = 0 then
		Tms.Assert (True, "Files match");
		Quit := True;
	    else
		Tms.Assert (False, "Files do not match ..." & Ascii.Lf &
				      Expected_File & Ascii.Lf & Actual_File);
	    end if;
	end if;
	Close_Em;
    exception
	when others =>
	    Tms.Assert (False,
			"Exception occurred reading files (expected none)");
	    Close_Em;
    end Text_File_Compare;
end Tms_Evals;

