--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
-----------------------------------------------------------------------------
-- 
--
-- This package defines:
--
-- o  Whether dynamic allocation is allowed;
-- o  Sizes of static structures to hold scorecard information
--
-- CUSTOMIZATION NOTES:
--
--     You can customize anything here to control memory allocation
--

package Tm_Mcdc_Config is

    pragma Pure;
    pragma Not_Elaborated;

    --
    -- This flag determines whether dynamic allocation may be used.
    --
    -- A value of "True" indicates that the memory allocator may be used
    -- to get more storage if the required storage exceeds the static
    -- sizes below.  A value of "False" means that only the static
    -- areas will be used.
    -- 
    -- Allowing dynamic allocation is more flexible, but may not be 
    -- desireable for some applications.  Generally, leave this at True
    -- except for some safety-critical Apex Embedded applications, or when
    -- the runtime does not support dynamic allocation.

    Allow_Dynamic_Allocation : constant Boolean := True;

    -- The value of this constant is determined by the total number
    -- of compilation units you will instrument at a time.  The worst case
    -- would be the total number of compilation units in your entire program.

    Maximum_Number_Of_Instrumented_Compilation_Units : constant := 50;


    -- To determine the value of this constant, analyze the code to 
    -- determine then number of "case" statements in TestMate 
    -- instrumented units.  Then to that result, add the number
    -- of decisions with more than 6 conditions.
    -- Needed only for MCDC.  If doing decision or statment coverage,
    -- this can be set to 1.  Never set this to 0, a constraint error will result.

    Maximum_Case_And_Complex_Decision_Statements : constant := 100;


    -- To determine the worst case value of this constant, analyze the 
    -- code to determine ALL of the possible values that may be presented 
    -- to ALL "case" in TestMate instrumented units.  Then locate all
    -- decisions with more than 6 conditions.  For each of these decisions,
    -- compute the maximum number of combinations of conditions
    -- (2 ** number_of_conditions). If a decision has 8 conditions, the
    -- maximum combinations is 2 ** 8 or 256.  Total the number of combinations
    -- for all decisions and add in the total "case" statement values.
    -- This is the "worst case" value for this constant.

    -- In reality, the value of this constant needs to be only large
    -- enough to hold the number of values and combinations that are
    -- tested in one test case.  Once a new test case is started with
    -- a call to Tm_Score_Manager.Start_Test_Case, the array used to
    -- hold the values and combinations is reused.

    -- For example, you have two "case" statements in your
    -- application.  One is a "case" on an enumeration type with
    -- 15 enumeration literals.  The second is a "case" on a subtype
    -- of integer with a range of 1..20.  This would make a total
    -- of 35 possible values for all "case" statements. 
    -- You also have three decisions with 9 conditions each this results
    -- in 3 * ( 2**9 ) = 1536 required for complex decisions.
    -- The worst case value would then be 35 + 1536.  Again, this
    -- assumes you will be testing all 35 value for the case statements
    -- and all 512 possible combinations of conditions for each of the
    -- three complex decisions in a single test case.
    -- Needed only for MCDC.  If doing decision or statment coverage,
    -- this can be set to 1.  Never set this to 0, a constraint error will result.

    Maximum_Number_Of_Values_For_All_Case_Statements : constant := 500;


    -- To determine the value of this constant, add all the values of the 
    -- 4th parameter on the function call Tm_Mcdc_Score.Register in each
    -- instrumented unit.  This constant is the size of the boolean bitmap
    -- used to score the conditions/entries/exits.

    Maximum_Number_Of_Conditions : constant := 3000;

end Tm_Mcdc_Config;
pragma Api (Testmate);
