-- Copyright (C) 1995,2000,2002 Rational Software Corporation.
-- All rights reserved.

-- RESTRICTED RIGHTS LEGEND
--
-- Use, duplication, or disclosure by the Government and Boeing is subject to
-- restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
-- Technical Data and Computer Software clause of DFAR 252.227-7013 and in
-- similiar clauses in the FAR and NASA FAR Supplement.
------------------------------------------------------------------------------
-- PROPRIETARY AND CONFIDENTIAL INFORMATION OF RATIONAL SOFTWARE CORP.,
-- USE OR COPYING WITHOUT EXPRESS WRITTEN AUTHORIZATION IS STRICTLY PROHIBITED.
-- THIS MATERIAL IS PROTECTED AS AN UNPUBLISHED WORK UNDER THE U.S. COPYRIGHT
-- ACT OF 1976.  CREATED 1995.  ALL RIGHTS RESERVED.
------------------------------------------------------------------------------
--
-- File Name: tm_mcdc_io_support.1.ada
--
-- All file manipulation operations are isolated in this one package to
-- simplify adaption to various targets.  A simple Unix version of this
-- package will utilize low level primitives to create/fill/close the
-- appropriate output file.  Embedded targets will likely need to
-- communicate across an ethernet or serial line to the host, which will
-- in turn create the output file.
--

package Tm_Mcdc_Io_Support is
    pragma Not_Elaborated;

    type Log_Msg_Kind is (Note_Msg, Warning_Msg, Error_Msg);

    type Object is private;

    procedure Create (The_Obj      : in out Object;
                      The_Id       : Integer;
                      Is_Scorecard : Boolean);
    --
    -- Create an output file for a scorecard file if IS_SCORECARD, otherwise
    -- a trace file is constructed.  How the output file name is determined
    -- depends on what kind of system is being used.  THE_ID is used to make
    -- the output file name unique, unless it is a negative number which will
    -- be ignored.
    --

    procedure Close (The_Obj : in out Object);
    --
    -- Close a created object THE_OBJ.
    --

    procedure Put_Line (The_Obj : Object; The_Line : String);
    --
    -- Add a line to THE_OBJ.  Raises PROGRAM_ERROR if THE_OBJ has not been
    -- created.
    --

    procedure Put_Message (The_Obj  : Object;
                           The_Line : String;
                           The_Kind : Log_Msg_Kind := Error_Msg);
    --
    -- Add a pass-through log message
    --

private

    type Object is
        record
            The_File : Integer;
        end record;
    pragma Convention (Ada_Pass_By_Reference, Object);

end Tm_Mcdc_Io_Support;
