-- Copyright (C) 1995,2000,2002 Rational Software Corporation.
-- All rights reserved.

-- RESTRICTED RIGHTS LEGEND
--
-- Use, duplication, or disclosure by the Government and Boeing is subject to
-- restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
-- Technical Data and Computer Software clause of DFAR 252.227-7013 and in
-- similiar clauses in the FAR and NASA FAR Supplement.
------------------------------------------------------------------------------
-- PROPRIETARY AND CONFIDENTIAL INFORMATION OF RATIONAL SOFTWARE CORP.,
-- USE OR COPYING WITHOUT EXPRESS WRITTEN AUTHORIZATION IS STRICTLY PROHIBITED.
-- THIS MATERIAL IS PROTECTED AS AN UNPUBLISHED WORK UNDER THE U.S. COPYRIGHT
-- ACT OF 1976.  CREATED 1995.  ALL RIGHTS RESERVED.
------------------------------------------------------------------------------
--
-- File Name: tm_mcdc_runtime_support.1.ada
--
-- Encapsulates the APEX specific runtime interface routines needed to
-- support MCDC trace functionality.
--

with System;

package Tm_Mcdc_Runtime_Support is
    pragma Not_Elaborated;

    type Trace_Integer is range System.Min_Int .. System.Max_Int;
    --
    -- Need an integer type that encompasses all possible values in the
    -- largest discrete type supported by the target.
    --

    generic
        with procedure Exit_Handler;
    procedure Register_Exit_Handler;
    --
    -- Register a routine that is to be called after execution of the main
    -- program has completed.

    package Allocator is
        -- Dynamic allocation services from the runtime
        function Alloc (Size_In_Bytes : Natural) return System.Address;
        procedure Free (Memory_Item : in System.Address);
    end Allocator;

    package Mcdc_String_Utilities is

        function Hash_String (S : String) return Integer;
        --
        -- Return a hash value for a string.
        --

        function Locate (Fragment : Character; Within : String) return Natural;
        --
        -- Returns the index value (in Within) of the first occurrance of
        -- Fragment, or 0 if none found.
        --

        function Reverse_Locate
                    (Fragment : Character; Within : String) return Natural;
        --
        -- Returns the index value (in Within) of the last occurrance of
        -- Fragment, or 0 if none found.
        --

        function Number_To_String (Value : Trace_Integer) return String;
        --
        -- Convert VALUE into a string in base 16.  The result is a minimal
        -- sized string.
        --

        function String_To_Number (Value : String) return Trace_Integer;
        --
        -- Convert VALUE into a TRACE_INTEGER.  VALUE is assumed to be a
        -- TRACE_INTEGER in base 16 representation.
        --

    end Mcdc_String_Utilities;

end Tm_Mcdc_Runtime_Support;
