-- Copyright (C) 1995,2000,2002 Rational Software Corporation.
-- All rights reserved.

-- RESTRICTED RIGHTS LEGEND
--
-- Use, duplication, or disclosure by the Government and Boeing is subject to
-- restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
-- Technical Data and Computer Software clause of DFAR 252.227-7013 and in
-- similiar clauses in the FAR and NASA FAR Supplement.
------------------------------------------------------------------------------
-- PROPRIETARY AND CONFIDENTIAL INFORMATION OF RATIONAL SOFTWARE CORP.,
-- USE OR COPYING WITHOUT EXPRESS WRITTEN AUTHORIZATION IS STRICTLY PROHIBITED.
-- THIS MATERIAL IS PROTECTED AS AN UNPUBLISHED WORK UNDER THE U.S. COPYRIGHT
-- ACT OF 1976.  CREATED 1995.  ALL RIGHTS RESERVED.
------------------------------------------------------------------------------
--
-- File Name: tm_mcdc_score.2.ada
--

with Tm_Mcdc_Score_Manager;
with Tm_Mcdc_Score_Support;
with Unchecked_Conversion;

pragma Elaborate (Tm_Mcdc_Score_Manager);

package body Tm_Mcdc_Score is

    pragma Suppress (Access_Check);
    pragma Suppress (Discriminant_Check);
    pragma Suppress (Index_Check);
    pragma Suppress (Length_Check);
    pragma Suppress (Range_Check);
    pragma Suppress (Storage_Check);

    package Msm renames Tm_Mcdc_Score_Manager;
    package Mss renames Tm_Mcdc_Score_Support;

    function "+" (L, R : Mss.Condition_Id) return Mss.Condition_Id
       renames Mss."+";
    function Is_Defined
                (L : Unit_Decisions; R : Unit_Decisions := null) return Boolean
       renames Mss."/=";
    function Is_Defined
                (L : Unit_Id; R : Unit_Id := Mss.Null_Unit_Id) return Boolean
       renames Mss."/=";


    type Acc_Condition_Id is access Mss.Condition_Id;
    for Acc_Condition_Id'Storage_Size use 0;
    function Convert is new Unchecked_Conversion
                               (System.Address, Acc_Condition_Id);


    ---------------------------------------------------------------------------
    -- Interface routines
    ---------------------------------------------------------------------------

    function Register (The_Unit                  : String;
                       The_Timestamp             : Integer;
                       The_Checksum              : Integer;
                       The_Decision_Array_Length : Natural;
                       The_Kind                  : Character := '#')
                      return Unit_Info is
    begin
        if Msm.Data_Collection_Is_On then
            declare
                Result : Unit_Info := Mss.Register
                                         (Msm.The_Active_Scorecard, The_Unit,
                                          The_Timestamp, The_Checksum,
                                          The_Decision_Array_Length, The_Kind);
            begin
                return Result;
            end;
        else
            return Mss.Null_Unit_Info;
        end if;
    end Register;


    function Decision1 (The_Expr       : Boolean;
                        The_Unit_Array : Unit_Decisions;
                        The_Offset     : Condition_Id) return Boolean is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (The_Unit_Array) then
            The_Unit_Array
               (Mss.Condition_Id (The_Offset) + Boolean'Pos (The_Expr)) := True;
        end if;
        return The_Expr;
    end Decision1;


    function Decision (The_Expr       : Boolean;
                       The_Unit_Array : Unit_Decisions;
                       The_Index      : System.Address;
                       The_Bias       : Condition_Id) return Boolean is
        The_Acc_Cond : Acc_Condition_Id := Convert (The_Index);
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (The_Unit_Array) then
            The_Unit_Array (Mss.Condition_Id (The_Bias) + The_Acc_Cond.all) :=
               True;
        end if;
        The_Acc_Cond.all := 0; -- must set to 0 for next decision
        return The_Expr;
    end Decision;


    function Decisionx (The_Expr  : Boolean;
                        In_Unit   : Unit_Id;
                        The_Line  : Natural;
                        The_Value : System.Address) return Boolean is
        The_Acc_Cond : Acc_Condition_Id := Convert (The_Value);
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (In_Unit) then
            Mss.Register_Decision
               (The_Score   => Msm.The_Active_Scorecard,
                The_Unit    => In_Unit,
                The_Line    => The_Line,
                The_Line_Id => 1,
                The_Result  => Mss.Trace_Integer (The_Acc_Cond.all));
            The_Acc_Cond.all := 0;
        end if;
        return The_Expr;
    end Decisionx;


    function Decisionx (The_Expr    : Boolean;
                        In_Unit     : Unit_Id;
                        The_Line    : Natural;
                        The_Line_Id : Natural;
                        The_Value   : System.Address) return Boolean is
        The_Acc_Cond : Acc_Condition_Id := Convert (The_Value);
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (In_Unit) then
            Mss.Register_Decision
               (The_Score   => Msm.The_Active_Scorecard,
                The_Unit    => In_Unit,
                The_Line    => The_Line,
                The_Line_Id => The_Line_Id,
                The_Result  => Mss.Trace_Integer (The_Acc_Cond.all));
            The_Acc_Cond.all := 0;
        end if;
        return The_Expr;
    end Decisionx;


    function Decisionp (The_Expr    : Boolean;
                        In_Unit     : Unit_Id;
                        The_Line    : Natural;
                        The_Line_Id : Natural := 1) return Boolean is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (In_Unit) then
            Mss.Register_Decision
               (The_Score   => Msm.The_Active_Scorecard,
                The_Unit    => In_Unit,
                The_Line    => The_Line,
                The_Line_Id => The_Line_Id,
                The_Result  => Mss.Trace_Integer (Boolean'Pos (The_Expr)));
        end if;
        return The_Expr;
    end Decisionp;


    function Decisionx (The_Expr : Trace_Integer;
                        In_Unit  : Unit_Id;
                        The_Line : Natural) return Trace_Integer is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (In_Unit) then
            Mss.Register_Decision (The_Score   => Msm.The_Active_Scorecard,
                                   The_Unit    => In_Unit,
                                   The_Line    => The_Line,
                                   The_Line_Id => 1,
                                   The_Result  => Mss.Trace_Integer (The_Expr));
        end if;
        return The_Expr;
    end Decisionx;


    function Decisionx (The_Expr    : Trace_Integer;
                        In_Unit     : Unit_Id;
                        The_Line    : Natural;
                        The_Line_Id : Natural) return Trace_Integer is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (In_Unit) then
            Mss.Register_Decision (The_Score   => Msm.The_Active_Scorecard,
                                   The_Unit    => In_Unit,
                                   The_Line    => The_Line,
                                   The_Line_Id => The_Line_Id,
                                   The_Result  => Mss.Trace_Integer (The_Expr));
        end if;
        return The_Expr;
    end Decisionx;


    procedure Decision (The_Unit_Array : Unit_Decisions;
                        The_Offset     : Condition_Id) is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (The_Unit_Array) then
            The_Unit_Array (Mss.Condition_Id (The_Offset)) := True;
        end if;
    end Decision;


    function Decision (The_Unit_Array : Unit_Decisions;
                       The_Offset     : Condition_Id) return Boolean is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (The_Unit_Array) then
            The_Unit_Array (Mss.Condition_Id (The_Offset)) := True;
        end if;
        return True;
    end Decision;


    procedure Decision (The_Unit_Array : Unit_Decisions;
                        The_Offset     : Condition_Id;
                        The_Cond       : out Condition_Id) is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (The_Unit_Array) then
            The_Unit_Array (Mss.Condition_Id (The_Offset)) := True;
        end if;
        The_Cond := 0;
    end Decision;


    procedure Decision_If (The_Unit_Array : Unit_Decisions;
                           The_Offset     : Condition_Id;
                           The_Cond       : in out Condition_Id) is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (The_Unit_Array) then
            if The_Cond = -1 then
                The_Unit_Array (Mss.Condition_Id (The_Offset)) := True;
                The_Cond                                       := 0;
            end if;
        end if;
    end Decision_If;


    function Decision_Simple (The_Expr       : Boolean;
                              The_Unit_Array : Unit_Decisions;
                              The_Offset     : Condition_Id) return Boolean is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (The_Unit_Array) then
            The_Unit_Array (Mss.Condition_Id (The_Offset)) := True;
        end if;
        return The_Expr;
    end Decision_Simple;


    function Decision_Simple (The_Expr       : Boolean;
                              The_Unit_Array : Unit_Decisions;
                              The_Offset     : Condition_Id;
                              If_Equal       : Boolean) return Boolean is
    begin
        if Msm.Data_Collection_Is_On and then Is_Defined (The_Unit_Array) then
            if The_Expr = If_Equal then
                The_Unit_Array (Mss.Condition_Id (The_Offset)) := True;
            end if;
        end if;
        return The_Expr;
    end Decision_Simple;


    function Condition1 (The_Expr : Boolean; The_Cond : System.Address)
                        return Boolean is
        The_Acc_Cond : Acc_Condition_Id;
    begin
        if Msm.Data_Collection_Is_On then
            if The_Expr then
                The_Acc_Cond     := Convert (The_Cond);
                The_Acc_Cond.all := The_Acc_Cond.all + 1;
            end if;
        end if;
        return The_Expr;
    end Condition1;


    function Condition2 (The_Expr : Boolean; The_Cond : System.Address)
                        return Boolean is
        The_Acc_Cond : Acc_Condition_Id;
    begin
        if Msm.Data_Collection_Is_On then
            if The_Expr then
                The_Acc_Cond     := Convert (The_Cond);
                The_Acc_Cond.all := The_Acc_Cond.all + 2;
            end if;
        end if;
        return The_Expr;
    end Condition2;


    function Condition3 (The_Expr : Boolean; The_Cond : System.Address)
                        return Boolean is
        The_Acc_Cond : Acc_Condition_Id;
    begin
        if Msm.Data_Collection_Is_On then
            if The_Expr then
                The_Acc_Cond     := Convert (The_Cond);
                The_Acc_Cond.all := The_Acc_Cond.all + 4;
            end if;
        end if;
        return The_Expr;
    end Condition3;


    function Condition4 (The_Expr : Boolean; The_Cond : System.Address)
                        return Boolean is
        The_Acc_Cond : Acc_Condition_Id;
    begin
        if Msm.Data_Collection_Is_On then
            if The_Expr then
                The_Acc_Cond     := Convert (The_Cond);
                The_Acc_Cond.all := The_Acc_Cond.all + 8;
            end if;
        end if;
        return The_Expr;
    end Condition4;


    function Condition5 (The_Expr : Boolean; The_Cond : System.Address)
                        return Boolean is
        The_Acc_Cond : Acc_Condition_Id;
    begin
        if Msm.Data_Collection_Is_On then
            if The_Expr then
                The_Acc_Cond     := Convert (The_Cond);
                The_Acc_Cond.all := The_Acc_Cond.all + 16;
            end if;
        end if;
        return The_Expr;
    end Condition5;


    function Condition6 (The_Expr : Boolean; The_Cond : System.Address)
                        return Boolean is
        The_Acc_Cond : Acc_Condition_Id;
    begin
        if Msm.Data_Collection_Is_On then
            if The_Expr then
                The_Acc_Cond     := Convert (The_Cond);
                The_Acc_Cond.all := The_Acc_Cond.all + 32;
            end if;
        end if;
        return The_Expr;
    end Condition6;


    function Condition (The_Expr  : Boolean;
                        The_Cond  : System.Address;
                        The_Index : Condition_Id) return Boolean is
        The_Acc_Cond : Acc_Condition_Id;
    begin
        if Msm.Data_Collection_Is_On then
            if The_Expr then
                The_Acc_Cond     := Convert (The_Cond);
                The_Acc_Cond.all := The_Acc_Cond.all +
                                       Mss.Condition_Id (The_Index);
            end if;
        end if;
        return The_Expr;
    end Condition;


end Tm_Mcdc_Score;
