-- Copyright (C) 1995-1996,1999-2002 Rational Software Corporation.
-- All rights reserved.

--
------------------------------------------------------------------------------
-- RESTRICTED RIGHTS LEGEND
--
-- Use, duplication, or disclosure by the Government and Boeing is subject to
-- restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
-- Technical Data and Computer Software clause of DFAR 252.227-7013 and in
-- similiar clauses in the FAR and NASA FAR Supplement.
------------------------------------------------------------------------------
-- PROPRIETARY AND CONFIDENTIAL INFORMATION OF RATIONAL SOFTWARE CORP.,
-- USE OR COPYING WITHOUT EXPRESS WRITTEN AUTHORIZATION IS STRICTLY PROHIBITED.
-- THIS MATERIAL IS PROTECTED AS AN UNPUBLISHED WORK UNDER THE U.S. COPYRIGHT
-- ACT OF 1976.  CREATED 1995.  ALL RIGHTS RESERVED.
------------------------------------------------------------------------------
--
-- File Name: tm_mcdc_score_manager.1.ada
--
-- Scorecard information needs to be flushed out of the runtime every so often
-- in order to support separate test cases in Ada test scripts.  Also desirable
-- to separate scorecard emission from normal analyzer interface.
--
-- The usage of this package is as follows:
-- 1.  The active scorecard object is maintained here.
-- 2.  The start and end of test cases is registered here so that scorecards
--     may be flushed out as appropriate.
-- 3.  An implicit test case is started with program elaboration and terminated
--     with program termination.
--

with System;
with Tm_Mcdc_Score_Support;

package Tm_Mcdc_Score_Manager is

    -- This package does need to be elaborated!
    -- It contains no global data objects, all scorecard related data is located
    -- in Tm_Mcdc_Score_Data.


    function The_Active_Scorecard return Tm_Mcdc_Score_Support.Scorecard_Ref;
    pragma Inline (The_Active_Scorecard);
    --
    -- Returns the scorecard object that all operations will directly utilize
    -- in order to register dynamic events.  The scorecard is a global to
    -- expediate usage.  (Danamic use of this scorecard will commonly slow down
    -- dynamic execution, so anything that can be done to speed up this usage
    -- is encouraged.
    --

    procedure Start_Test_Case;
    --
    -- Start a new test case.  The Data in THE_ACTIVE_SCORECARD is cleared out
    -- so that this test case starts out fresh.
    --

    procedure End_Test_Case (The_Test_Case_Id : Natural);
    --
    -- End the currect test case.  The data from THE_ACTIVE_SCORECARD is flushed
    -- and the scorecard data is cleared.  The name of the scorecard file is
    -- augmented with the specified THE_TEST_CASE_ID.
    --

    function Data_Collection_Is_On return Boolean;
    pragma Inline (Data_Collection_Is_On);
    --
    -- Predicate to determine whether data collection is on.
    -- Data collection may be temporarily turned off.
    -- At the end of program execution data collection is permanently turned off
    -- while the scorecard is written out.
    --

end Tm_Mcdc_Score_Manager;
pragma Api (Testmate);
