-- Copyright (C) 1995,2000,2002 Rational Software Corporation.
-- All rights reserved.

-- RESTRICTED RIGHTS LEGEND
--
-- Use, duplication, or disclosure by the Government and Boeing is subject to
-- restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
-- Technical Data and Computer Software clause of DFAR 252.227-7013 and in
-- similiar clauses in the FAR and NASA FAR Supplement.
------------------------------------------------------------------------------
-- PROPRIETARY AND CONFIDENTIAL INFORMATION OF RATIONAL SOFTWARE CORP.,
-- USE OR COPYING WITHOUT EXPRESS WRITTEN AUTHORIZATION IS STRICTLY PROHIBITED.
-- THIS MATERIAL IS PROTECTED AS AN UNPUBLISHED WORK UNDER THE U.S. COPYRIGHT
-- ACT OF 1976.  CREATED 1995.  ALL RIGHTS RESERVED.
------------------------------------------------------------------------------

with Tm_Mcdc_Score_Data;
with Tm_Mcdc_Io_Support;
with Tm_Mcdc_Runtime_Support;
with Tm_Mcdc_Score_Support;
with Unchecked_Conversion;

package body Tm_Mcdc_Score_Manager is

    pragma Suppress (Access_Check);
    pragma Suppress (Discriminant_Check);
    pragma Suppress (Index_Check);
    pragma Suppress (Length_Check);
    pragma Suppress (Range_Check);

    package Mrs renames Tm_Mcdc_Runtime_Support;
    package Mss renames Tm_Mcdc_Score_Support;

    ---------------------------------------------------------------------------
    -- Support routines
    ---------------------------------------------------------------------------

    function Image (The_Value : Integer) return String is
    begin
        return Mrs.Mcdc_String_Utilities.Number_To_String
                  (Mrs.Trace_Integer (The_Value));
    end Image;


    function Image (The_Value : Mss.Unit_Id) return String is
        function As_Integer is new Unchecked_Conversion
                                      (Mss.Unit_Id, Mrs.Trace_Integer);
        The_Integer_Value : Mrs.Trace_Integer := As_Integer (The_Value);
    begin
        if Mrs."<" (The_Integer_Value, 0) then
            The_Integer_Value :=
               Mrs."-" (The_Integer_Value, Mrs.Trace_Integer'First);
        end if;
        return Mrs.Mcdc_String_Utilities.Number_To_String (The_Integer_Value);
    end Image;


    function Image (The_Value : Mss.Bool_Decision_Array) return String is
    begin
        if The_Value'Length > 0 then
            return Mss.Convert (The_Value);
        else
            return "0";
        end if;
    end Image;


    procedure Process_Scorecard (The_Id : Integer := -1) is

        The_Score_File : Tm_Mcdc_Io_Support.Object;

        procedure Print_Unit (The_Unit           : String;
                              The_Id             : Mss.Unit_Id;
                              The_Timestamp      : Integer;
                              The_Checksum       : Integer;
                              The_Decision_Array : Mss.Bool_Decision_Array;
                              The_Kind           : Character) is
        begin
            Tm_Mcdc_Io_Support.Put_Line
               (The_Score_File, The_Kind & The_Unit & ' ' & Image (The_Id) &
                                   ' ' & Image (The_Timestamp) &
                                   ' ' & Image (The_Checksum) & ' ' &
                                   Image (The_Decision_Array));
        end Print_Unit;

        procedure Print_Score (The_Unit    : Mss.Unit_Id;
                               The_Line    : Natural;
                               The_Line_Id : Natural;
                               The_Trace   : String) is
        begin
            if The_Line_Id <= 1 then
                Tm_Mcdc_Io_Support.Put_Line
                   (The_Score_File, Image (The_Unit) & ' ' &
                                       Image (The_Line) & ' ' & The_Trace);
            else
                Tm_Mcdc_Io_Support.Put_Line
                   (The_Score_File, Image (The_Unit) & ' ' &
                                       Image (The_Line) & '-' &
                                       Image (The_Line_Id) & ' ' & The_Trace);
            end if;
        end Print_Score;

    begin
        Tm_Mcdc_Score_Data.Coverage_Collection_Disabled := True;

        -- Open the scorecard file ...
        Tm_Mcdc_Io_Support.Create
           (The_Score_File, The_Id, Is_Scorecard => True);

        -- Dump errors ...
        if The_Id < 1 then
            declare
                procedure Print_Warning (The_Msg : String) is
                begin
                    Tm_Mcdc_Io_Support.Put_Message
                       (The_Score_File, The_Msg,
                        Tm_Mcdc_Io_Support.Warning_Msg);
                end Print_Warning;

                procedure Dump is new Mss.Dump_Warnings (Print_Warning);
            begin
                Dump;
            end;
        end if;

        -- Walk the unit list and output the results ...
        declare
            procedure Register is new Mss.Iterate (Print_Unit, Print_Score);
        begin
            Register (The_Active_Scorecard);
        end;

        -- Dump statistics ...
        declare
            procedure Print_Msg (The_Stat : String) is
            begin
                Tm_Mcdc_Io_Support.Put_Line (The_Score_File, The_Stat);
            end Print_Msg;

            procedure Dump is new Mss.Dump_Statistics (Print_Msg);
        begin
            Dump;
        end;

        -- Close the generated scorecard file ...
        Tm_Mcdc_Io_Support.Close (The_Score_File);

        Tm_Mcdc_Score_Data.Coverage_Collection_Disabled := False;
    exception
        when others =>
            Tm_Mcdc_Score_Data.Coverage_Collection_Disabled := False;
            -- ??? Just keep silent for now ...
    end Process_Scorecard;


    procedure Process_Default_Scorecard is
    begin
        --if not Mss.Is_Clean_Scorecard (Tm_Mcdc_Score_Data.The_Scorecard) then
        Process_Scorecard (-1);
        -- clear termination scorecard data ...
        --end if;
    end Process_Default_Scorecard;


    ---------------------------------------------------------------------------
    -- Interface routines
    ---------------------------------------------------------------------------

    function The_Active_Scorecard return Tm_Mcdc_Score_Support.Scorecard_Ref is
    begin
        return Tm_Mcdc_Score_Support.Reference
                  (Tm_Mcdc_Score_Data.The_Scorecard'Address);
    end The_Active_Scorecard;

    procedure Start_Test_Case is
    begin
        Mss.Clean_Scorecard (Tm_Mcdc_Score_Data.The_Scorecard);
    end Start_Test_Case;


    procedure End_Test_Case (The_Test_Case_Id : Natural) is
    begin
        if not Mss.Is_Clean_Scorecard (Tm_Mcdc_Score_Data.The_Scorecard) then
            Process_Scorecard (The_Test_Case_Id);
        end if;
    end End_Test_Case;

    function Data_Collection_Is_On return Boolean is
    begin
        return not Tm_Mcdc_Score_Data.Coverage_Collection_Disabled;
    end Data_Collection_Is_On;

begin
    Tm_Mcdc_Score_Data.Coverage_Collection_Disabled := True;

    declare
        procedure Register_Close is
           new Tm_Mcdc_Runtime_Support.Register_Exit_Handler
                  (Process_Default_Scorecard);
    begin
        Tm_Mcdc_Score_Data.Coverage_Collection_Disabled := True;
        Register_Close;
    end;

    Tm_Mcdc_Score_Data.Coverage_Collection_Disabled := False;
end Tm_Mcdc_Score_Manager;
