--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
with Apex.Unix_Operations;

package body Tm_Simple_Io is

    Line_Terminator : constant Character := Ascii.Lf;

    function "=" (L, R : Apex.Unix_Types.Int) return Boolean
       renames Apex.Unix_Types."=";

    procedure Put (Char : Character; Fildes : Apex.Unix_Types.Int := Stdout) is
        Stat : Apex.Unix_Types.Ssize_T;
        X    : aliased constant Character := Char;
    begin
        Stat := Apex.Unix_Operations.Write
                   (Fildes, Apex.Unix_Types.To_Char_Ptr (X'Address), 1);
    end Put;

    procedure Put (Str : String; Fildes : Apex.Unix_Types.Int := Stdout) is
        Stat : Apex.Unix_Types.Ssize_T;
    begin
        Stat := Apex.Unix_Operations.Write
                   (Fildes, Apex.Unix_Types.To_Char_Ptr (Str'Address),
                    Str'Length);
    end Put;

    procedure Put_Line (Str : String; Fildes : Apex.Unix_Types.Int := Stdout) is
    begin
        Put (Str, Fildes);
        New_Line (1, Fildes);
    end Put_Line;

    procedure New_Line (Count  : Integer             := 1;
                        Fildes : Apex.Unix_Types.Int := Stdout) is
    begin
        for I in 1 .. Count loop
            Put (Line_Terminator, Fildes);
        end loop;
    end New_Line;


    procedure Get (Ch     : out Character;
                   Eof    : out Boolean;
                   Fildes : Apex.Unix_Types.Int := Stdin) is
        C      : aliased Character;
        N_Read : Apex.Unix_Types.Ssize_T;
        use Apex.Unix_Types;
    begin
        N_Read := Apex.Unix_Operations.Read
                     (Fildes, Apex.Unix_Types.To_Char_Ptr (C'Address), 1);
        -- n_read = 0 means EOF...
        Eof    := N_Read = 0;
        Ch     := C;
    end Get;


    procedure Get (Str                    : out String;
                   Eof                    : out Boolean;
                   Strip_Line_Terminators : in Boolean          := True;
                   Fildes                 : Apex.Unix_Types.Int := Stdin) is
        C    : Character := Ascii.Nul;
        Xeof : Boolean   := False;
    begin
        for I in Str'Range loop
            while not Xeof and (C = Line_Terminator and
                                Strip_Line_Terminators) loop
                Get (C, Xeof, Fildes);
            end loop;
            Str (I) := C;
            exit when Xeof;
        end loop;
        Eof := Xeof;
    end Get;


    procedure Get_Line (Str    : out String;
                        Last   : out Natural;
                        Eof    : out Boolean;
                        Fildes : Apex.Unix_Types.Int := Stdin) is
        C    : Character;
        Xeof : Boolean := False;
    begin
        Last := 0;
        for I in Str'Range loop
            Get (C, Xeof, Fildes);
            exit when C = Line_Terminator or Xeof;
            Str (I) := C;
            Last    := I;
        end loop;
        Eof := Xeof;
    end Get_Line;

end Tm_Simple_Io;
