--+--------------------------------------------------------------------------+
--| IBM Confidential                                                         |
--| OCO Source Materials                                                     |
--| Rational Apex                                                            |
--| (C) Copyright IBM Corp. 2008 All Rights Reserved                         |
--|                                                                          |
--| The source code for this program is not published or otherwise divested  |
--| of its trade secrets, irrespective of what has been deposited with the   |
--| U.S. Copyright Office.                                                   |
--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
with Apex.Unix_Types;

package Tms_Arguments is

    --[internal]
    --[internal] It would be much nicer to have a communications record that simply
    --[internal] has an entry for each call type.  Because most of the TMS routines
    --[internal] use strings for parameters, passing multiple unconstrained values
    --[internal] in a record that is supposed to handle several different program
    --[internal] signatures becomes very unruly.  So, this package is the compromise.
    --
    -- All signatures are encoded into a byte array in the following format:
    --
    --    <procedure-id> & <p1-size> & <p1> & <p2-size> & <p2> ...
    --
    -- where p1 and p2 are parameter one and two respectively, the <pX-size>
    -- entries are a fixed two bytes in length, and any number of parameters
    -- can be added to the array.
    --
    -- Encoding of a parameters signature is performed by concatenation of 
    -- argument byte arrays by the caller.  Decoding is performed through
    -- an initial decode call followed by a sequence of calls to get the 
    -- various paramters.  The decoder must know the order and the type of
    -- the parameters in an encoded byte array.
    --

    type Byte_Array is array (Natural range <>) of Apex.Unix_Types.Char;
    pragma Convention (Ada_Pass_By_Reference, Byte_Array);

    type Routine_Names is
       (None, -- a null request; ignored
              -- Package TMS routines
        Tms_Register_Result, Tms_Assert, Tms_Check_Assertions,
        Tms_Traverse_Assertions, Tms_Number_Of_Tests,
        Tms_Prior_Test_Case_Result, Tms_Get_Test_Case_Result, Tms_Put_Line,
        Tms_Set_Log_Filter, Tms_Get_Switch, Tms_Register_Env,
        -- support routines
        Defer_Results, Resolve_Placeholder, Test_List_Initialize,
        Start_Test_Case, Start_Driver, End_Driver, End_Test_Case,
        Coverage_Pre_Run, Coverage_Post_Run, Getenv);

    --
    -- Encoding routines : generic returns "<param-size> & <param>"
    --
    function Encode_Routine (Routine : Routine_Names) return Byte_Array;

    generic
        type Parameter_Type is private;
    function Encode (Param : Parameter_Type) return Byte_Array;

    function Encode_String (Param : String) return Byte_Array;

    --
    -- Decoding routines
    --
    function Routine (From_Call : Byte_Array) return Routine_Names;
    function Parameter_Count (From_Call : Byte_Array) return Natural;

    generic
        type Parameter_Type is private;
        Nil_Parameter_Type : Parameter_Type;
    function Decode (From_Call : Byte_Array; Parameter : Natural)
                    return Parameter_Type;

    function Decode_String
                (From_Call : Byte_Array; Parameter : Natural) return String;

end Tms_Arguments;
--[api] pragma Api (Testmate);
