--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
with Tms_Script_Commands;

package Tms_Script_Support is


    Script_Halt : exception;

    function Resolve_Placeholders (In_The_String : String) return String;
    --
    -- Resolve any placeholders found In_The_String.
    --
    -- Precondition  : Initialize and Next_test_Case has been called
    -- Postcondition : 
    -- Return value  : In_The_String with any embedded placeholders expanded
    --


    --
    -- The following are synchronization points with the test execution
    -- server and the coverage analyzer.
    --
    -- The sequencing must be:
    --          Initialize
    --          foreach test case loop
    --            Start_Test_Case
    --            - PROLOG
    --            - PRECONDITION
    --            Start_Driver
    --            - DRIVER
    --            End_Driver
    --            - EVALUATION_ACTION
    --            - EPILOG
    --            End_Test_Case
    --          exception =>
    --            when Early_Term => End_Test_Case
    --          end loop
    --
    -- At any point, early termination of the TC (e.g. timeout) may be 
    -- indicated by setting the tc termination reason, and raising
    -- Early_Term.  
    --

    procedure Initialize;
    --
    -- Initialize package and obtain test list controls from Test Execution
    -- Server.  Test Execution Server synchs start of test list with script.
    --
    -- Precondition  :
    -- Postcondition : Test list controls are retrieved


    generic
	type Test_Case_Ids is (<>);
	with procedure Call_Prolog (Id : Test_Case_Ids);
	with function Call_Precondition (Id : Test_Case_Ids) return Boolean;
	with procedure Call_Test_Driver (Id : Test_Case_Ids);
	with procedure Call_Evaluation_Action (Id : Test_Case_Ids);
	with procedure Call_Epilog (Id : Test_Case_Ids);
    procedure Test_Invoke (Id : Test_Case_Ids);


    Module_Information : constant String :=
       "@(#) $Id: tms_script_support.1.ada,v 1.1 09/21/2000 23:01:05 gsmith Exp $\";

end Tms_Script_Support;

--[api] pragma Api (Testmate);  

