--+--------------------------------------------------------------------------+
--| IBM Confidential                                                         |
--| OCO Source Materials                                                     |
--| Rational Apex                                                            |
--| (C) Copyright IBM Corp. 2008 All Rights Reserved                         |
--|                                                                          |
--| The source code for this program is not published or otherwise divested  |
--| of its trade secrets, irrespective of what has been deposited with the   |
--| U.S. Copyright Office.                                                   |
--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2003 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
with Apex.Unix_Operations;
with Apex.Unix_Types;

with Tms_Arguments;
with Tms_Communications;
with Tms_Communications_Client;
with Tms_Script_Commands;
with Tms_Link_Contribution;

package body Tms is

    package Ta renames Tms_Arguments;
    package Tc renames Tms_Communications;
    package Tcc renames Tms_Communications.Communicate;
    package Client renames Tms_Communications_Client;

    function "&" (L, R : Ta.Byte_Array) return Ta.Byte_Array renames Ta."&";

    --[internal] --
    --[internal] -- Most of these instantiations should just be exported straight out
    --[internal] -- of Tms_Arguments, but there is a strange compiler bug that does
    --[internal] -- not allow that, soo ...
    --[internal] --
    function Encode_String (Param : String) return Ta.Byte_Array
       renames Ta.Encode_String;
    function Encode_Natural is new Ta.Encode (Natural);
    function Encode_Log_Msg_Kind is new Ta.Encode (Log_Msg_Kind);
    function Encode_Boolean is new Ta.Encode (Boolean);
    function Encode_Filter is new Ta.Encode (Log_Filter);
    function Encode_Log_Mode is new Ta.Encode (Log_Mode);

    function Decode_String
                (From_Call : Ta.Byte_Array; Parameter : Natural) return String
       renames Ta.Decode_String;
    function Decode_Natural is new Ta.Decode (Natural, 0);
    function Decode_Boolean is new Ta.Decode (Boolean, False);
    function Decode_Assertion_Status is new Ta.Decode (Assertion_Status, None);


    -----------------------------------------------------------------------
    -- Exported
    -----------------------------------------------------------------------

    function Pass return Test_Result is
    begin
        return "PASS";
    end Pass;

    function Fail return Test_Result is
    begin
        return "FAIL";
    end Fail;

    function Unknown return Test_Result is
    begin
        return "UNKNOWN";
    end Unknown;

    function Failed_Assertion return Test_Result is
    begin
        return "FAILED_ASSERTION";
    end Failed_Assertion;

    function Failed_Precondition return Test_Result is
    begin
        return "FAILED_PRECONDITION";
    end Failed_Precondition;

    function Fail_With_Exception return Test_Result is
    begin
        return "FAIL_WITH_EXCEPTION";
    end Fail_With_Exception;

    function Terminated return Test_Result is
    begin
        return "TERMINATED";
    end Terminated;

    function Timeout_Cpu return Test_Result is
    begin
        return "TIMEOUT_CPU";
    end Timeout_Cpu;

    function Timeout_Elapsed return Test_Result is
    begin
        return "TIMEOUT_ELAPSED";
    end Timeout_Elapsed;

    function Unhandled_Signal return Test_Result is
    begin
        return "UNHANDLED_SIGNAL";
    end Unhandled_Signal;

    function Not_Applicable return Test_Result is
    begin
        return "NOT_APPLICABLE";
    end Not_Applicable;

    function Is_Equal (L, R : Test_Result) return Boolean is
        function Upper_Case (S : String) return String is
            Local : String (S'First .. S'Last) := S;
        begin
            for I in Local'First .. Local'Last loop
                if Local (I) in 'a' .. 'z' then
                    Local (I) := Character'Val (Character'Pos (Local (I)) - 32);
                end if;
            end loop;
            return Local;
        end Upper_Case;
    begin
        -- must be a case insensitive compare!!!
        return Upper_Case (L) = Upper_Case (R);
    end Is_Equal;

    procedure Register_Result (The_Result        : Tms.Test_Result;
                               For_The_Test_Case : String := "") is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Register_Result) &
              Encode_String (The_Result) & Encode_String (For_The_Test_Case);
    begin
        -- Write to parent (and return) ...
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array :=
               Tcc.Receive; -- keep sync'ed with server
        begin
            null;
        end;
    exception
        when others =>
            Tc.Report_Error ("Could not register result" & The_Result);
            if not Client.Bastard then
                raise;
            end if;
    end Register_Result;


    procedure Assert (Condition     : Boolean;
                      Description   : String   := "";
                      Log_Entry_For : Log_Mode := Default_Log_Mode) is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Assert) & Encode_Boolean (Condition) &
              Encode_String (Description) & Encode_Log_Mode (Log_Entry_For);
    begin
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array :=
               Tcc.Receive; -- keep sync'ed with server
        begin
            null;
        end;
    exception
        when others =>
            Tc.Report_Error ("Could not verify assertion " & Description);
            Tc.Report_Error (" (assertion condition was " &
                             Boolean'Image (Condition) & ')');
            if not Client.Bastard then
                raise;
            end if;
    end Assert;


    function Check_Assertions return Assertion_Status is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Check_Assertions);
    begin
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array := Tcc.Receive;
        begin
            return Decode_Assertion_Status (Response, 1);
        end;
    exception
        when others =>
            Tc.Report_Error ("Could not check assertion status");
            if not Client.Bastard then
                raise;
            end if;
    end Check_Assertions;


    -- generic
    --  with procedure Visit (Description : String; Value : Boolean);
    procedure Traverse (Only_Visit_Failures : Boolean := False) is
        -- procedure Visit_All is new Assertions.Traverse (Visit);
        -- Remote parameters are:
        --     Only_Visit_Failures : Boolean
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Traverse_Assertions) &
              Encode_Boolean (Only_Visit_Failures);
    begin
        Client.Send (Request);
        -- keep getting responses until False in first param...
        loop
            declare
                Response : constant Ta.Byte_Array := Tcc.Receive;
            begin
                exit when not Decode_Boolean (Response, 1);
                Visit (Decode_String (Response, 2),
                       Decode_Boolean (Response, 3));
            end;
        end loop;
    exception
        when others =>
            Tc.Report_Error ("Could not traverse assertions");
            if not Client.Bastard then
                raise;
            end if;
    end Traverse;

    function Resolve_Placeholders (In_The_String : String) return String is
    begin
        return Tms_Script_Commands.Resolve_Placeholders (In_The_String);
    exception
        when others =>
            Tc.Report_Error ("Could not resolve placeholders in """ &
                             In_The_String & '"');
            if Client.Bastard then
                return In_The_String;
            else
                raise;
            end if;
    end Resolve_Placeholders;



    function Value (Of_The_Switch : String) return String is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Get_Switch) &
              Encode_String (Of_The_Switch);
    begin
        -- Write to parent and get response
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array :=
               Tcc.Receive; -- keep sync'ed with server
        begin
            return Decode_String (Response, 1);
        end;
    exception
        when others =>
            Tc.Report_Error ("Could not get value of switch " & Of_The_Switch);
            if Client.Bastard then
                return "";
            else
                raise;
            end if;
    end Value;



    function Number_Of_Tests (With_Result : Test_Result) return Natural is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Number_Of_Tests) &
              Encode_String (Resolve_Placeholders (With_Result));
    begin
        -- Write to parent and read up response ...
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array := Tcc.Receive;
        begin
            return Decode_Natural (Response, 1);
        end;
    exception
        when others =>
            Tc.Report_Error ("Could not get obtain Number_Of_Tests; using 0");
            if Client.Bastard then
                return 0;
            else
                raise;
            end if;
    end Number_Of_Tests;



    function Prior_Test_Case_Result return Test_Result is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Prior_Test_Case_Result);
    begin
        -- Write to parent and wait for response ...
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array := Tcc.Receive;
        begin
            return Decode_String (Response, 1);
        end;
    exception
        when others =>
            Tc.Report_Error
               ("Could not get value of Prior_Test_Case_Result; returning UNKNOWN");
            if Client.Bastard then
                return Tms.Unknown;
            else
                raise;
            end if;
    end Prior_Test_Case_Result;


    function The_Test_Case_Result (Of_Test_Case : String) return Test_Result is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Get_Test_Case_Result) &
              Encode_String (Resolve_Placeholders (Of_Test_Case));
    begin
        -- Write to parent and read up response ...
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array := Tcc.Receive;
        begin
            return Decode_String (Response, 1);
        end;
    exception
        when others =>
            Tc.Report_Error
               ("Could not get value of The_Test_Case_Result for test case " &
                Of_Test_Case & "; returning UNKNOWN");
            if Client.Bastard then
                return Tms.Unknown;
            else
                raise;
            end if;
    end The_Test_Case_Result;



    procedure Register_Environment is
        type Acc_String is access String;
        type String_Array is array (Natural range <>) of Acc_String;
        Env     : Apex.Unix_Types.Char_Ptr_Ptr := Apex.Unix_Operations.Environ;
        Count   : constant Natural             := Apex.Unix_Types.Count (Env);
        All_Env : String_Array (1 .. Count)    := (others => null);
        This    : Natural                      := 0;

        procedure Visit (Cptr : Apex.Unix_Types.Char_Ptr;
                         Quit : in out Boolean) is
        begin
            This           := This + 1;
            All_Env (This) := new String'(Apex.Unix_Types.To_String (Cptr));
        exception
            when Constraint_Error =>
                Put_Line ("Internal: failure registering environment", At_Msg);
        end Visit;

        procedure Visit_Env is new Apex.Unix_Types.Visit_Strings (Visit);

        function Encode_String_Array
                    (The_Array : String_Array) return Ta.Byte_Array is
        begin
            if The_Array'First = The_Array'Last then
                return Encode_String (The_Array (The_Array'First).all);
            else
                return Encode_String (The_Array (The_Array'First).all) &
                          Encode_String_Array (The_Array (The_Array'First + 1 ..
                                                             The_Array'Last));
            end if;
        end Encode_String_Array;
    begin
        Visit_Env (Env);
        -- Write to parent and return
        declare
            Request : constant Ta.Byte_Array :=
               Ta.Encode_Routine (Ta.Tms_Register_Env) &
                  Encode_Natural (Count) & Encode_String_Array (All_Env);
        begin
            -- Apex should clean up the dynamic memory for us, since acc_string
            -- is local scope
            -- Write to parent and read up response ...
            Client.Send (Request);
            declare
                Response : constant Ta.Byte_Array := Tcc.Receive;
            begin
                null;
            end;
        exception
            when others =>
                Tc.Report_Error ("Could not Register_Environment");
                if not Client.Bastard then
                    raise;
                end if;
        end;
    end Register_Environment;



    procedure Put_Line (Message : String; Kind : Log_Msg_Kind := Text_Msg) is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Put_Line) &
              Encode_String (Resolve_Placeholders (Message)) &
              Encode_Log_Msg_Kind (Kind);
    begin
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array :=
               Tcc.Receive; -- keep sync'ed with server
        begin
            null;
        end;
    exception
        when others =>
            Tc.Report_Error (Message);
            if not Client.Bastard then
                raise;
            end if;
    end Put_Line;


    procedure Set_Log_Filter (Filter : Log_Filter) is
        Request : constant Ta.Byte_Array :=
           Ta.Encode_Routine (Ta.Tms_Set_Log_Filter) & Encode_Filter (Filter);
    begin
        Client.Send (Request);
        declare
            Response : constant Ta.Byte_Array :=
               Tcc.Receive; -- keep sync'ed with server
        begin
            null;
        end;
    exception
        when others =>
            Tc.Report_Error ("Could not Set_Log_Filter");
            if not Client.Bastard then
                raise;
            end if;
    end Set_Log_Filter;


    function Connected return Boolean is
    begin
        if not Client.Connected then
            -- Attempt connection first
            Client.Connect;
        end if;
        return (not Client.Bastard) and Client.Connected;
    end Connected;

    function Hex_Dump
                (Source : System.Address; Bit_Count : Integer) return String is
        type Short_U is mod 2 ** System.Storage_Unit;
        for Short_U'Size use System.Storage_Unit;

        Src : array (0 .. (Bit_Count - 1) / System.Storage_Unit) of Short_U;
        for Src'Address use Source;
        pragma Import (Ada, Src);

        Result    : String (1 .. Src'Last * 2 + 2);
        R_Index   : Natural                               := Result'First;
        Su        : Natural;
        Hex_Digit : constant array (0 .. 15) of Character :=
           ('0', '1', '2', '3', '4', '5', '6', '7',
            '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');
    begin
        if System.Storage_Unit /= 8 then
            -- This algorithm assumes storage is in 8-bit units
            raise Constraint_Error;
        end if;

        for I in Src'Range loop
            Su                   := Natural (Src (I));
            Result (R_Index)     := Hex_Digit (Su / 2 ** 4);
            Result (R_Index + 1) := Hex_Digit (Su mod 2 ** 4);
            R_Index              := R_Index + 2;
        end loop;
        return Result;
    end Hex_Dump;

    function Image (X : System.Address) return String is
        Addressible_X : aliased System.Address := X;
        use System;
    begin
        if X = System.Null_Address then
            return "System.Null_Address";
        end if;
        return Hex_Dump (Addressible_X'Address, System.Address'Size);
    end Image;

    function Image (X : Float) return String is
        -- Underlying algorithm is quite large -- we'll just engineer a
        -- call to a function available in the runtime.
        function Rt_Float_Image (X             : Float;
                                 Fore          : Natural;
                                 Aft           : Natural;
                                 Exp           : Natural;
                                 Leading_Space : Boolean) return String;
        pragma Import (Ada, Rt_Float_Image, Link_Name => "__FLOAT_IMAGE");
    begin
        return Rt_Float_Image (X, 2, Float'Digits, 3, False);
    exception
        when others =>
            return "(invalid float)";
    end Image;

    function Image (X : Long_Float) return String is
        function Rt_Float_Image (X             : Long_Float;
                                 Fore          : Natural;
                                 Aft           : Natural;
                                 Exp           : Natural;
                                 Leading_Space : Boolean) return String;
        pragma Import (Ada, Rt_Float_Image, Link_Name => "__LONG_FLOAT_IMAGE");
    begin
        return Rt_Float_Image (X, 2, Long_Float'Digits, 3, False);
    exception
        when others =>
            return "(invalid float)";
    end Image;

    function Image (X : Short_Float) return String is
        function Rt_Float_Image (X             : Float;
                                 Fore          : Natural;
                                 Aft           : Natural;
                                 Exp           : Natural;
                                 Leading_Space : Boolean) return String;
        pragma Import (Ada, Rt_Float_Image, Link_Name => "__FLOAT_IMAGE");
    begin
        return Rt_Float_Image (Float (X), 2, Short_Float'Digits, 3, False);
    exception
        when others =>
            return "(invalid float)";
    end Image;

    function Image (X : String) return String is
        -- Make an Ada-like image of the string
        -- NUL characters are skipped
        Result  : String (1 .. 2 + X'Last * 2);
        R_Index : Natural := Result'First + 1;
    begin
        Result (1) := '"';
        for I in X'Range loop
            Result (R_Index) := X (I);
            R_Index          := R_Index + 1;

            if X (I) = '"' then
                Result (R_Index) := '"';
                R_Index          := R_Index + 1;
            end if;

            if X (I) = Ascii.Nul then
                R_Index := R_Index - 1;
            end if;
        end loop;

        Result (R_Index) := '"';
        return Result (1 .. R_Index);
    end Image;

    function Image (X : Calendar.Time) return String is
        Sec : Calendar.Day_Duration := Calendar.Seconds (X);
        Hrs : Natural               := Natural (Natural (Sec) / 3600);
        Min : Natural               :=
           Natural (Sec - (Duration (3600.00) * Hrs)) / 60;

        function Two_Digit (X : Integer) return String is
            Im : constant String := Integer'Image (X);
        begin
            if Im'Length = 2 then
                return '0' & Im (Im'Last);
            end if;
            return Im (Im'Last - 1 .. Im'Last);
        end Two_Digit;

        function Rt_Float_Image (X             : Float;
                                 Fore          : Natural;
                                 Aft           : Natural;
                                 Exp           : Natural;
                                 Leading_Space : Boolean) return String;
        pragma Import (Ada, Rt_Float_Image, Link_Name => "__FLOAT_IMAGE");
        use Calendar;
    begin
        Sec := Sec - (3600.00 * Hrs) - (60.0 * Min);
        if Sec < 10.0 then
            return Year_Number'Image (Year (X)) & '.' & Two_Digit (Month (X)) &
                      '.' & Two_Digit (Day (X)) & '.' & Two_Digit (Hrs) &
                      ':' & Two_Digit (Min) & ':' & '0' &
                      Rt_Float_Image (Float (Sec), 1, 4, 0,
                                      Leading_Space => False);
        end if;
        return Year_Number'Image (Year (X)) & '.' & Two_Digit (Month (X)) &
                  '.' & Two_Digit (Day (X)) & '.' &
                  Two_Digit (Hrs) & ':' & Two_Digit (Min) & ':' &
                  Rt_Float_Image (Float (Sec), 2, 4, 0, Leading_Space => False);
    end Image;

    function Hex_Image (X : Integer) return String is
        Addressible_X : aliased Integer := X;
    begin
        return Hex_Dump (Addressible_X'Address, Addressible_X'Size);
    end Hex_Image;

    function Hex_Image (X : Long_Integer) return String is
        Addressible_X : aliased Long_Integer := X;
    begin
        return Hex_Dump (Addressible_X'Address, Addressible_X'Size);
    end Hex_Image;

end Tms;
