--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
-- 
-- File Name: tms_script_commands.1.ada
--
-- Use script_commands in place of direct use of tms_script_support in the 
-- tower so that the TRIGGER_EVENT routine is not needed by the tower.  If
-- tower packages use tms_script_support directly then can't run with CA.
-- Tms_Script_Support should only be directly with'ed by generated test 
-- scripts!!!
-- 
with System;

package Tms_Script_Commands is


    --
    -- The execution server informs the script when the overall
    -- execution should be abandoned and why.
    --
    type Termination_Reason is
       (None, Elapsed_Time_Overflow, Cpu_Time_Overflow,  
	Max_Number_Of_Failures_Exceeded,
	Prior_Unhandled_Exception, Internal_Error, Other_Error);

    --
    -- ... however, the script informs the execution server how each
    -- test case terminated.  The script will first call TMS to 
    -- log the error reason.
    --
    type Tc_Termination_Reason is
       (None, Failed_Precondition, Missing_Io_File, Elapsed_Time_Overflow,
	Cpu_Time_Overflow, Unhandled_Exception, Internal_Error, Other_Error);

    type String_Handle is access String;
    type Seconds is range System.Min_Int .. System.Max_Int;


    procedure Defer_Results;
    --
    -- This tells the execution server that the results will not be 
    -- immediately forthcoming, and instead the test list should be 
    -- terminated early.  At some later time the user will run 
    -- "testmate process_results raw_results_file" to register the 
    -- results information.
    -- 


    function Resolve_Placeholders (In_The_String : String) return String;
    --
    -- Resolve any placeholders found In_The_String.
    --
    -- Precondition  : Initialize and Next_test_Case has been called
    -- Postcondition : 
    -- Return value  : In_The_String with any embedded placeholders expanded
    --


    function Coverage_Pre_Run return String;
    --
    -- Inform the coverage analyzer that the script has been linked and
    -- is ready to run.
    --


    procedure Coverage_Post_Run;
    --
    -- Inform the coverage analyzer that the script has been run and
    -- is ready to processed.
    --


    function Getenv (The_Variable : String) return String;
    --
    -- Retrieves the value of the environment variable THE_VARIABLE from
    -- TestMate environment state.
    --


    --
    -- The following routines are located here so that they can be 
    -- called both by Tms_Script_Support and Tms_Main ("tms" and
    -- "tms_support" program) without forcing Tms_Main to bring
    -- in the TRIGGER_EVENT symbols defined in Tms_Script_Support.
    --
    procedure Initialize;
    function Initialize return Natural;

    procedure Start_Test_Case;
    procedure Start_Test_Case (Input_File     : in out String_Handle;
			       Output_File    : in out String_Handle;
			       Error_File     : in out String_Handle;
			       Exec_Context   : in out String_Handle;
			       Allow_Coverage : in out Boolean;
			       Cpu_Limit      : in out Seconds;
			       Elapsed_Limit  : in out Seconds;
			       Term_Reason    : in out Termination_Reason);



    procedure End_Test_Case;
    procedure End_Test_Case (Term_Reason : in Tc_Termination_Reason);

    procedure Start_Driver;

    procedure End_Driver;


end Tms_Script_Commands;

--[api] pragma Api (Testmate);  
