--+--------------------------------------------------------------------------+
--| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
--| This software contains proprietary and confidential information of       |
--| Rational and its suppliers. Use, disclosure or reproduction is           |
--| prohibited without the prior express written consent of Rational.        |
--+--------------------------------------------------------------------------+
--
with Tms;
package Tms_Evals is

    --
    -- This package contains some miscellaneous evaluation utilities 
    -- that can be used for Ada test cases.
    --
    -- These utilities all perform some sort of check and then register 
    -- the result as one or more assertions.  Thus, these functions may 
    -- be freely ganged (i.e., run one after another) to produce a test
    -- result.  
    --
    -- Note that a call to Tms.Register_Result call will override any
    -- previous assertions.
    --


    procedure Gang_Result (The_Result : Tms.Test_Result);
    --
    -- Same as Tms.Register_Result, but takes intermediate assertions into
    -- account.  Failed assertions will prevent a result of PASS from
    -- being registered.
    --


    procedure File_Compare (Expected : String := "<expected_output:f>";
			    Actual   : String := "<output:f>");
    --
    -- Compares the two files and asserts True if the files are exactly
    -- identical, False if they are not.  Intended usage is within the 
    -- evaluation action.
    --
    -- By default, the test case's standard output is compared to the
    -- expected output file declared in the test case.
    --


    procedure Text_File_Compare (Expected : String := "<expected_output:f>";
				 Actual   : String := "<output:f>");
    --
    -- File comparison when both files are known to be text.  Compares the 
    -- two text files and asserts True if the files are exactly
    -- identical, line by line, or False if they are not.  Intended usage 
    -- is within the evaluation action.
    --
    -- The primary distinction from File_Compare is that mismatches are 
    -- reported line by line, instead of byte by byte.  By default, the 
    -- test case's standard output is compared to the expected output file 
    -- declared in the test case.
    --

end Tms_Evals;
--[api] pragma Api (Testmate);
